/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.ticket;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by The eXo Platform SARL
 * Author : Chung Nguyen
 *          nguyenchung136@yahoo.com
 * Jan 5, 2006
 * 
 * This class is used to perform a task or access a  resource, after the  TicketService 
 * has checked the security  and validation of  the accessor. The properties, type of the
 * ticket should be persisted  with the Ticket info  and reconstruct before the method execute
 * is  invoked. 
 * 
 * The third party developer or  other service can  customized this command to perform the 
 * task he wants or access a resource while delegate the security burden to the TicketService.
 */
public abstract class TicketCommand {
  private Ticket ticket_ ;
  private Map<String, String> properties_  ;
  /**
   * The ticket that associates with this this ticket command.
   * @return
   */
  public Ticket  getTicket() { return ticket_ ; }
  /**
   * Set the ticket that associate with this ticket command. Usually , this method is invoked
   * in the TicketService before  the execute themod is called.
   * @param ticket
   */
  public void    setTicket(Ticket ticket) { ticket_ = ticket ; }
  
  /**
   * The developer can store some property into the command and later access the property
   * in the  execute method.  For example , when you share a document , you need to store the
   * path of the document  in the properties. When you access the ticket and get the document
   * the execute method will get the path of the document from the properties , load the document
   * and return
   * @param name  the property name
   * @return return  the propety that matched the name or null
   */
  final public String getProperty(String name) {
    if(properties_ == null)  return null ;
    return properties_.get(name) ; 
  }
  /**
   * Store  the a property into the properties map.
   * @param name  the name of the properties.
   * @param value
   */
  final public void   setProperty(String name, String value) {
    if(properties_ == null) properties_ = new HashMap<String, String>() ;
    properties_.put(name, value) ; 
  }
  
  /**
   * TODO  What is this for ???????
   * @param requiredPermissions
   */
  final public void setPropertys(String[] requiredPermissions){
    if(properties_==null) return ;
    for(int i=0;i<requiredPermissions.length;i++){
      properties_.put("requiredPermissions["+i+"]",requiredPermissions[i]);
    }
  }
  
  /**
   * TODO  What is this for ???????
   * @param requiredPermissions
   */
  final public String[] getPropertys(String[] name){
    if(properties_==null) return null;
    String[] strProperty=new String[name.length];
    for(int i=0;i<name.length;i++){
      strProperty[i]=properties_.get(name[i].toString());
    }
    return strProperty;
  }
  
  final public Map<String, String> getProperties() { return properties_ ; }
  final public void setProperties(Map<String, String> props) { properties_ = props ; }
  
  final public String  removeProperty(String name) {  return properties_.remove(name) ; }
  
  abstract public Object execute()  throws Exception ;
  
  /**
   * The type of the command
   * @return
   * @throws Exception
   */
  abstract public String getType() throws Exception ;
}
