/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.templates.groovy.impl;

import groovy.lang.Writable;
import groovy.text.SimpleTemplateEngine;
import groovy.text.Template;

import java.io.InputStream;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.templates.groovy.GroovyTemplateLoader;
import org.exoplatform.services.templates.groovy.GroovyTemplateService;
import org.exoplatform.services.templates.groovy.TemplateNotFoundException;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Dec 26, 2005
 */
public class GroovyTemplateServiceImpl implements GroovyTemplateService {
  static TemplateLoaderPriorityComparator LOADER_COMPARATOR = new TemplateLoaderPriorityComparator() ;
  
  private SimpleTemplateEngine engine_ ;
  private List<GroovyTemplateLoader> templateLoader_ ;
  private ExoCache templatesCache_ ;
  private boolean cacheTemplate_ = true ;
  
  public GroovyTemplateServiceImpl(InitParams params, 
                                   CacheService cservice) throws Exception {
    engine_ = new SimpleTemplateEngine() ;
    templateLoader_ =  new ArrayList<GroovyTemplateLoader>() ;
    templatesCache_ = cservice.getCacheInstance(GroovyTemplateService.class.getName()) ;
    cacheTemplate_ = "true".equals(params.getValueParam("cache.template").getValue()) ;
  }
  
  public void addTemplateLoader(ComponentPlugin plugin) {
    GroovyTemplateLoader loader = (GroovyTemplateLoader) plugin ;
    addTemplateLoader(loader) ;
  }

  synchronized public void addTemplateLoader(GroovyTemplateLoader loader) {
    templateLoader_.add(loader) ;
    Collections.sort(templateLoader_, LOADER_COMPARATOR) ;
  }
  
  public String merge(String type, String name, Map binding) 
  throws TemplateNotFoundException, Exception {
    try {
      Template template = getTemplate(type, name) ;
      if(template == null) {
        throw new Exception("Cannot  find the template " + template) ;
      }
      binding.put("service", this) ;
      binding.put("binding" , binding) ;
      String s = template.make(binding).toString() ;
      return s ;
    } finally {
      binding.remove("service") ;
      binding.remove("binding") ;
    }
  }

  public void merge(Writer writer, String type, String tname, Map binding)
  throws TemplateNotFoundException, Exception {
    try {
      Template template = getTemplate(type, tname) ;
      binding.put("service", this) ;
      binding.put("binding", binding) ;
      Writable writable = template.make(binding) ;
      writable.writeTo(writer) ;
    } finally {
      binding.remove("service") ;
      binding.remove("binding") ;
    }
  }
  
  public String include(String name, Map binding) throws TemplateNotFoundException, Exception  {
    Map newBinding = new HashMap() ;
    if(binding != null)  newBinding.putAll(binding) ;
    Template template = getTemplate(null, name) ;
    return template.make(binding).toString() ;
  }
  
  public String include(String type, String name, Map binding) throws TemplateNotFoundException, Exception  {
    Map newBinding = new HashMap() ;
    if(binding != null)  newBinding.putAll(binding) ;
    Template template = getTemplate(type, name) ;
    return template.make(binding).toString() ;
  }
  
  private Template getTemplate(String type, String name) 
  throws TemplateNotFoundException, Exception { 
    Template template = (Template)templatesCache_.get(name) ;
    if(template != null)  return template ;
    for(GroovyTemplateLoader loader : templateLoader_){
      if(loader.canLoad(type, name)) {
        InputStream is = loader.load(name) ;
        byte[]  buf = new byte[is.available()] ;
        is.read(buf) ;
        String text =  new String(buf) ;
        template = engine_.createTemplate(text) ;
        if(cacheTemplate_) templatesCache_.put(name, template) ;
      }
    }
    if(template == null) {
      throw new TemplateNotFoundException("Cannot find template " + name) ;
    }
    return template ;
  }
  
  static public class TemplateLoaderPriorityComparator implements Comparator<GroovyTemplateLoader>  {
    
    public int compare(GroovyTemplateLoader loader1, GroovyTemplateLoader loader2) {
      return loader1.getPriority() - loader2.getPriority();
    }
    
  }
}
