/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.log;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.List;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.component.BaseComponentPlugin;
import org.exoplatform.portal.log.PageLogData;
import org.exoplatform.portal.log.Query;
import org.exoplatform.services.cache.CacheListener;
import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.cache.FIFOExoCache;
import org.exoplatform.services.database.DBObjectPageList;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.ObjectQuery;
import org.hibernate.Session;

/**
 * @author Hung Nguyen (nqhungvn@users.sourceforge.net)
 * @since Sep 26, 2005
 * @version $Id$
 */
public class PageActivitiesLogPlugin extends BaseComponentPlugin implements CacheListener {  
  private FIFOExoCache pageActivitiesLogCache_ ;
  private List<Object> logsToSave_  = new LinkedList<Object>();
  private int currentTimeIndex_ ;
  private Date currentTimeSlot_ ;
  private String uniqueId_ ; 
  private HibernateService hservice_ ;
  private CacheService cservice_ ;  
  
  public PageActivitiesLogPlugin(HibernateService hservice, CacheService cservice) throws Exception {
    hservice_ = hservice ;
    cservice_ = cservice ;
    pageActivitiesLogCache_ = (FIFOExoCache)cservice_.getCacheInstance("PageActivitiesLogCache") ;
    pageActivitiesLogCache_.addCacheListener(this) ;
    updateTime() ;
  }
  
  public void updateTime() throws Exception{
    GregorianCalendar  calendar = new  GregorianCalendar() ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    currentTimeSlot_ = calendar.getTime() ;
    currentTimeIndex_ = calendar.get(GregorianCalendar.HOUR_OF_DAY) ;    
    uniqueId_ = "/" + String.valueOf(calendar.get(GregorianCalendar.YEAR)) + 
                "-" + String.valueOf(calendar.get(GregorianCalendar.MONTH)+1) +
                "-" + String.valueOf(calendar.get(GregorianCalendar.DATE)) +
                "/" + String.valueOf(currentTimeIndex_);    
    if(!pageActivitiesLogCache_.isEmpty()) {
      synchronized(logsToSave_) {
        logsToSave_.addAll(pageActivitiesLogCache_.getCachedObjects()) ;    
        pageActivitiesLogCache_.clearCache() ;
      }
    }    
  }
  
  public int getCurrentTimeIndex() { return currentTimeIndex_ ; }
  public void logPageAccess(String pageId, String pageOwner, long exeTime, boolean error) {
    getOrCreatePageLogData(pageId, pageOwner, exeTime, error);    
  }
  
  private void getOrCreatePageLogData(String pageId, String pageOwner, long exeTime, boolean error) {
    try {
      PageLogData impl = 
        (PageLogData) pageActivitiesLogCache_.get(pageId+ uniqueId_) ;
      if(impl == null) {
        impl = new PageLogData() ;
        impl.logPageAccess(pageId + uniqueId_, pageId, pageOwner, currentTimeSlot_, 
            currentTimeIndex_, exeTime, error) ;
        pageActivitiesLogCache_.put(pageId+ uniqueId_, impl) ;
      }else {
        impl.updateLogAccess(exeTime, error) ;
      }
    } catch(Exception ex) {
      ex.printStackTrace() ;      
    }
  }
  
  synchronized public void savePageLogData(PageLogData data) throws Exception {
    Session session = hservice_.openSession() ;
    PageLogData obj = 
      (PageLogData) session.get(PageLogData.class, data.getId()) ;
    if(obj == null) {
      obj = data ;
      session.save(obj) ;
    } else {
      obj.merge(data) ;
      session.update(obj) ;
    }
    session.flush() ;
  }
  
  public PageList getPageLogDatas(Query q) throws Exception {
    ObjectQuery oq = new ObjectQuery(PageLogData.class);
    oq.addGT("timeSlot", q.getFromTimeSlot()) ;
    oq.addLT("timeSlot", q.getToTimeSlot()) ; 
    oq.addGT("accessCounter", q.getCounter()) ;
    String componentId = q.getComponentId() ; 
    if(!componentId.equals("") || componentId.length() > 0) 
      oq.addLIKE("pageOwner", componentId) ;
    if(q.getOrder() != "" && q.getOrder().equals("Desc")){
      oq.setDescOrderBy("timeSlot") ;
    }else if(q.getOrder() != "" && q.getOrder().equals("Asc")){
      oq.setAscOrderBy("timeSlot") ;
    }
    return new DBObjectPageList(hservice_, oq);
  }
  
  public List getPageLogChartDatas(Query q) throws Exception {
    ObjectQuery oq = new ObjectQuery(PageLogData.class);
    oq.addSelect("pageId") ;
    oq.addSUM("accessCounter") ;
    oq.addSUM("accessErrorCounter") ;
    oq.addSUM("summaryAccessTime") ;
    if(q.getMostAccess() != "" && q.getMostAccess().equals("most24")) {
      GregorianCalendar calendar = new GregorianCalendar() ;
      oq.addLT("timeSlot", calendar.getTime()) ;
      int dateBefore = calendar.get(GregorianCalendar.DATE) - 1;
      calendar.set(GregorianCalendar.DATE, dateBefore) ;
      oq.addGT("timeSlot", calendar.getTime()) ;
    }else if (q.getMostAccess() != "" && q.getMostAccess().equals("overall")){
      GregorianCalendar calendar = new GregorianCalendar() ;
      oq.addLT("timeSlot", calendar.getTime()) ;
    }else {
      oq.addGT("timeSlot", q.getFromTimeSlot()) ;
      oq.addLT("timeSlot", q.getToTimeSlot()) ; 
    }    
    oq.addGT("accessCounter", q.getCounter()) ;
    String componentId = q.getComponentId() ; 
    if(!componentId.equals("") || componentId.length() > 0) 
      oq.addLIKE("pageOwner", componentId) ;
    oq.setGroupBy("pageId") ;    
    String groupQuery = oq.getHibernateGroupByQuery() ;
    if(q.getMostAccess() != "")
      groupQuery += " order by sum(o.accessCounter) desc " ;
    Session session = hservice_.openSession() ;
    return session.createQuery(groupQuery).list() ;    
  }
  
  public void  saveMonitorData() throws Exception {
    List cacheObjectList = pageActivitiesLogCache_.removeCachedObjects() ;
    for(int i = 0; i < cacheObjectList.size(); i++){
      PageLogData data = (PageLogData)cacheObjectList.get(i) ;
      savePageLogData(data) ;
    }
  }
  
  public void savePrevTimeSlotMonitorData() throws Exception {
    if(logsToSave_.size() == 0)  return  ;
    List<Object> list = null ;
    synchronized (logsToSave_) {
      list =  new ArrayList<Object>(logsToSave_) ;
      logsToSave_.clear() ;
    }
    for(int i = 0; i < list.size(); i++){
      PageLogData data = (PageLogData) list.get(i) ;
      savePageLogData(data) ;
    }    
  }
  
  public void onExpire(ExoCache cache, Serializable key, Object value) throws Exception {
    PageLogData data = (PageLogData) value ;
    savePageLogData(data) ; 
  }
  
  public void onRemove(ExoCache cache, Serializable key, Object value) throws Exception {  }
  
  public void onPut(ExoCache cache, Serializable key, Object value) throws Exception { }
  
  public void onGet(ExoCache cache, Serializable key, Object value) throws Exception { }
  
  public void onClearCache(ExoCache cache) throws Exception { }  
}