/***************************************************************************
 *  Copyright 2001-2003 The eXo Platform SARL         All rights reserved. *
 *  Please look at license.txt in info directory for more license detail.  *
 ***************************************************************************
 */
package org.exoplatform.portlet.commons;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import javax.portlet.*;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.portal.session.RequestInfo;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.portletcontainer.ExoPortletContext;
import org.exoplatform.services.portletcontainer.impl.portletAPIImp.PortletContextImpl;
import org.exoplatform.services.portletcontainer.impl.portletAPIImp.PortletSessionImp;
import org.exoplatform.services.portletcontainer.impl.portletAPIImp.helpers.CustomRequestWrapper;
import org.exoplatform.services.portletcontainer.impl.portletAPIImp.helpers.CustomResponseWrapper;
/**
 * Created by The eXo Platform SARL        .
 *
 * @author Alexander Kravchuk
 * @version $Id: $
 */
public class CommonPortletBridge extends GenericPortlet {

  private static Log log;
  private String contextPath;
  protected String defaultPage;

  public static String PARAM_PREFIX = "portlet:";

  public void init(PortletConfig config) throws PortletException {
      super.init(config);
      PortalContainer manager = PortalContainer.getInstance();
      LogService logService =
              (LogService) manager.getComponentInstanceOfType(LogService.class);
    try {
        logService.setLogLevel("org.exoplatform.portlet.commons",
                               LogService.DEBUG, true);
    } catch (Exception ex) {
    }
      log = logService.getLog(this.getClass());
      //i hate build-in logger
      log = LogFactory.getLog(this.getClass());


      contextPath = config.getPortletContext().getPortletContextName();
      log.debug("context Path ["+contextPath+"]");

      defaultPage = config.getInitParameter("default-page");

  }

  public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
    log.debug( "$$$$$$$$$$$$$$$$ begin dispatch controller $$$$$$$$$$$$$$$$$$$$$$$$" );
    //wrap request & response
    CustomRequestWrapper requestWrapper = (CustomRequestWrapper)
                     ((HttpServletRequestWrapper)request).getRequest();
    CustomResponseWrapper responseWrapper =
            (CustomResponseWrapper) ((HttpServletResponseWrapper)response).getResponse();

    boolean isSharedSessionEnable = false;

    try {
      //read contexts
      PortletContextImpl portletContext = (PortletContextImpl) getPortletContext();
      ServletContext scontext = portletContext.getWrappedServletContext();
      ExoPortletContext context = (ExoPortletContext) request.getPortletSession().
                                  getPortletContext();
      isSharedSessionEnable = context.isSessionShared();
      log.debug("Session is shared " + isSharedSessionEnable);

      //get real request url
      String requestUrl = getRequestURL(requestWrapper);

      //get request dispatcher
      RequestDispatcher dispatcher = null;
      dispatcher = scontext.getRequestDispatcher(requestUrl) ;
      log.debug("RequestDispatcher is null? "+(dispatcher == null));

      //change request settings
      requestWrapper.setRedirected(true);
      requestWrapper.setRedirectedPath(
            requestUrl);
      requestWrapper.servletPath = requestUrl;
      requestWrapper.pathInfo = "";
      requestWrapper.contextPath = "/"+contextPath;
//      requestWrapper.servletPath = "/";
//      requestWrapper.pathInfo = requestUrl;
//            requestUrl);
      if(isSharedSessionEnable){
        PortletSessionImp pS = (PortletSessionImp)request.getPortletSession();
        requestWrapper.setSharedSession(pS.getSession());
        requestWrapper.setContextPath(request.getContextPath());
      }


      //!!!!!!!!!!Dispatch request !!!!!!!!!!!!!!!!!!!!!!
      dispatcher.include(requestWrapper, responseWrapper);
      //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      log.debug("Content-Type "+responseWrapper.getContentType());

      //read output
      InputStream responseAsInputStream;
      try{
        responseWrapper.getWriter();
        responseAsInputStream = new ByteArrayInputStream(
                      String.valueOf(responseWrapper.getPortletContent()).
                      getBytes());

      }
      catch (IllegalStateException ex){
        //if output writed to outputStream
        log.debug("getWriter can't be called, call getOutputStream instead" );


        responseAsInputStream = new ByteArrayInputStream(responseWrapper.toByteArray());
        //reset response wrapper
        responseWrapper.fillResponseWrapper(
            (HttpServletResponse) responseWrapper.getResponse());
      }

      ByteArrayOutputStream  byteOutput = new ByteArrayOutputStream();

      //set transform params
      PortletOutputTransformer rewriter  = new PortletOutputTransformer();
      rewriter.portalURI = getPorlalBaseURI();
      rewriter.portletURI = getPorletBaseURI(requestWrapper);
      rewriter.portalContextPath = requestWrapper.getContextPath();
      rewriter.portalQueryString = "";
      rewriter.paramNamespace = getPortletBasedParamName("");

      //transform output
      rewriter.rewrite(responseAsInputStream,byteOutput);

      //reset response
      responseWrapper.reset();

      //convert output of transformation to String
      //result of transformation is in "UTF-8"
      java.io.Reader reader = new java.io.InputStreamReader(
                  new ByteArrayInputStream(byteOutput.toByteArray()),"UTF-8");
      char[] buf = new char[1024];
      int readed;

      //write transformed output to response
      while ((readed = reader.read(buf)) > 0 ){
          responseWrapper.getWriter().write(buf,0,readed);
      }

    }

    catch (Exception e) {
      log.error("Error on dispatch request ",e);
      throw new PortletException("Problems occur when using PortletDispatcher", e);
    } finally {
      if(requestWrapper != null)
        requestWrapper.setRedirected(false);
      if(isSharedSessionEnable){
        requestWrapper.setSharedSession(null);
      }

    }
    log.debug( "$$$$$$$$$$$$$$$$ end dispatch controller $$$$$$$$$$$$$$$$$$$$$$$$" );



  }

  public void render(RenderRequest request, RenderResponse response) throws PortletException, java.io.IOException {
    doView(request, response);
  }


  /**
   * Generate full param name related to context of
   *  current portlet
   */
  private String getPortletBasedParamName(String shortName){
      return PARAM_PREFIX + contextPath + ":" + shortName;
  }


  private String getPorletBaseURI(HttpServletRequest request) {
      String result = getRequestURL(request);

      if (result.indexOf('/') != -1) {
          result = result.substring(0,result.lastIndexOf('/'));
      }


      RequestInfo rinfo = (RequestInfo)
                          SessionContainer.getComponent(RequestInfo.class);
      log.debug("------------ begin request info --------------");
      log.debug("RequestInfo.getOwnerURI() "+  rinfo.getOwnerURI());
      log.debug("request.getPathInfo() " + request.getPathInfo());
      log.debug("request.getServletPath() "+ request.getServletPath());
      log.debug("request.getQueryString() "+ request.getQueryString());
      log.debug("request.getContextPath() "+ request.getContextPath());
      log.debug("------------ end request info --------------");

      return result;
  }

  private String getRequestURL(HttpServletRequest request){
      String paramURL = getPortletBasedParamName("url");
      String result = request.getParameter(paramURL);
      result = result == null ? defaultPage : result;

      log.debug("request.getParameter('"+paramURL+"') ["+result+"]");
      return result;
  }

  //URI (path + query string) of current string
  private String getPorlalBaseURI(){
      RequestInfo rinfo =
            (RequestInfo)SessionContainer.getComponent(RequestInfo.class);
      String path = rinfo.getOwnerURI();
      return path;
  }


}


