/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import java.util.Collection;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.Query;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIListUser.java,v 1.18 2004/10/22 14:23:30 tuan08 Exp $
 */
public class UIListUser extends UIExoCommand {
  static String ADD_USER_ACTION = "addUser".intern() ;
  static Parameter[] ADD_USER_PARAM = { new Parameter(ACTION , ADD_USER_ACTION) } ;
	static Parameter[] VIEW_PROFILE = { new Parameter(ACTION , "viewProfile") } ;
	static Parameter[] VIEW_INFO =   { new Parameter(ACTION , "viewUserInfo")  } ;
	static Parameter[] DELETE =  { new Parameter(ACTION , DELETE_ACTION) } ;
  static Parameter[] REFRESH = { new Parameter(ACTION , "refresh") } ;
	
	private UIPageListIterator uiPageIterator_ ;
	private boolean adminRole_ ;
	private OrganizationService service_ ;
  private Query lastQuery_ ;
	
	public UIListUser(OrganizationService service) throws Exception {
		setId("UIListUser") ;
    setRendererType("VelocityRenderer") ;
		service_ = service  ;
		uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ; 
    adminRole_ = hasRole("admin") ;
		getChildren().add(uiPageIterator_) ;
    addListener();
    search(new Query()) ;    
	}
  
  public void addListener() throws Exception {
    addActionListener(AddUserActionListener.class, ADD_USER_ACTION) ;
    addActionListener(DeleteUserActionListener.class, DELETE_ACTION) ;
    addActionListener(ViewProfileActionListener.class, "viewProfile") ;
    addActionListener(ViewUserInfoActionListener.class, "viewUserInfo") ;
    addActionListener(RefreshListUserActionListener.class, "refresh") ;
    
  }
  
  public Parameter[] getAddUserParams() { return ADD_USER_PARAM ; }; 
  public Parameter[] getDeleteParams() { return DELETE ; };
  public Parameter[] getProfileParams() { return VIEW_PROFILE ; };
  public Parameter[] getInfoParams() { return VIEW_INFO ; };
  public Parameter[] getRefreshParams() { return REFRESH ; };  
   
  public Collection getUserList() throws Exception {
    return uiPageIterator_.getPageListDataHandler().getDatas();    
  }
   
	public void search(Query query) throws Exception {
    lastQuery_ = query ;
		uiPageIterator_.setPageList(service_.getUserHandler().findUsers(query)) ;
	}
  
  public void refresh() throws Exception {
    uiPageIterator_.setPageList(service_.getUserHandler().findUsers(lastQuery_)) ;
  }
	
	public boolean hasAdminRole() { return adminRole_ ; }
  
  static public class AddUserActionListener extends ExoActionListener  {
    public AddUserActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIListUser uiListUser = (UIListUser) event.getComponent() ;
      UIAccountForm accForm =  (UIAccountForm)uiListUser.getSibling(UIAccountForm.class) ;
      accForm.customizeNewAccountFromUserList() ;
      uiListUser.setRenderedSibling(UIAccountForm.class) ;
    }
  }
	
	static public class ViewUserInfoActionListener extends ExoActionListener  {
    public ViewUserInfoActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
		public void execute(ExoActionEvent event) throws Exception {
      UIListUser uiListUser = (UIListUser) event.getComponent() ;
			String userName = event.getParameter(OBJECTID) ;
			UIUserInfo uiUserInfo = 
				(UIUserInfo)uiListUser.getSibling(UIUserInfo.class) ;
			uiUserInfo.changeUser(userName) ;
			uiListUser.setRenderedSibling(UIUserInfo.class) ;
		}
	}
	
	static public class ViewProfileActionListener extends ExoActionListener  {
		public void execute(ExoActionEvent event) throws Exception {
      UIListUser uiListUser = (UIListUser) event.getComponent() ;
			String userName = event.getParameter(OBJECTID);
			UIUserProfileSummary uiUserProfile = 
				(UIUserProfileSummary)uiListUser.getSibling(UIUserProfileSummary.class) ;
			uiUserProfile.setUserProfile(userName) ;
			uiListUser.setRenderedSibling(UIUserProfileSummary.class) ;
		}
	}
	
  static public class RefreshListUserActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIListUser uiListUser = (UIListUser) event.getComponent() ;
      uiListUser.refresh() ;
    }
  }
  
	static public class DeleteUserActionListener extends ExoActionListener  {
    public DeleteUserActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
		public void execute(ExoActionEvent event) throws Exception {
      UIListUser uiListUser = (UIListUser) event.getComponent() ;
			String userName = event.getParameter(OBJECTID) ;
			uiListUser.service_.getUserHandler().removeUser(userName, true) ;
			uiListUser.refresh() ;
		}
	}
}