/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.commons.exception.ExoMessageException;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIMembershipForm.java,v 1.11 2004/10/22 14:23:30 tuan08 Exp $
 */
public class UIMembershipForm extends UISimpleForm {
  private static String USERNAME = "username" , MEMBERSHIP_NAME = "membershipName" ,
                        GROUP_ID = "groupId" ;
  static String SELECT_USER_ACTION = "selectUser".intern() ;
  static Parameter[] SELECT_USER_PARAM = { new Parameter(ACTION , SELECT_USER_ACTION) } ;
  
  public UIMembershipForm() throws Exception {
    super("membershipForm",  "post", null) ;
    setId("UIMembershipForm");
    setRendererType("SimpleFormVelocityRenderer") ;
    add(new UIStringInput(USERNAME, "")) ;
    add(new UISelectBox(MEMBERSHIP_NAME, "", null)) ;
    add(new UIStringInput(GROUP_ID, "")) ;
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(SelectUserListener.class,SELECT_USER_ACTION);
  }
  public Parameter[] getSelectUserParams() { return SELECT_USER_PARAM ; }; 
  public void populateFormByGroup(String groupId) throws Exception {
    populateCommon() ;
    UIStringInput groupIdField = getUIStringInput(GROUP_ID) ;
    groupIdField.setValue(groupId) ;
    groupIdField.setEditable(false) ; 
    getUIStringInput(USERNAME).setValue("") ;
  }
  
  public void populateFormByUser(String userName) throws Exception {
    populateCommon() ;
    UIStringInput usernameField = getUIStringInput(USERNAME) ;
    usernameField.setValue(userName) ;
    usernameField.setEditable(false) ; 
    getUIStringInput(GROUP_ID).setValue("") ;
  }
  
  private void populateCommon() throws Exception {
    OrganizationService service =
      (OrganizationService) PortalContainer.getComponent(OrganizationService.class);
    Collection memberships = service.getMembershipTypeHandler().findMembershipTypes() ;
    Iterator i = memberships.iterator() ;
    List roleOptions = new ArrayList(10) ;
    while(i.hasNext()) {
      MembershipType mt = (MembershipType) i.next();
      String name = mt.getName() ;
      roleOptions.add(new SelectItem(name, name)) ;
    }
    UISelectBox sfield = getUISelectBox(MEMBERSHIP_NAME) ;
    sfield.setValue("") ;
    sfield.setOptions(roleOptions) ; 
  }
  
  static public class SaveActionListener extends ExoActionListener  {
    public SaveActionListener( ) {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
  	public void execute(ExoActionEvent event) throws Exception {
      UIMembershipForm uiForm = (UIMembershipForm) event.getSource() ;
      OrganizationService service =
        (OrganizationService) PortalContainer.getComponent(OrganizationService.class);
  		String userName = uiForm.getUIStringInput(USERNAME).getValue() ;
      User user = service.getUserHandler().findUserByName(userName) ;
      if(user == null) {
        Object[] args = { userName } ;
        throw new ExoMessageException("UIMembershipForm.user-not-found", args) ;
      }
      String groupId = uiForm.getUIStringInput(GROUP_ID).getValue();
      Group group = service.getGroupHandler().findGroupById(groupId) ;
      if(group == null) {
        Object[] args = { groupId } ;
        throw new ExoMessageException("UIMembershipForm.group-not-found", args) ;
      }
      MembershipType mt = service.getMembershipTypeHandler().createMembershipTypeInstance() ;
      mt.setName( uiForm.getUISelectBox(MEMBERSHIP_NAME).getValue()) ;
//  		Membership membership = service.getMembershipHandler().createMembershipInstance();
//      membership.setMembershipType( uiForm.getUISelectBox(MEMBERSHIP_NAME).getValue());
      service.getMembershipHandler().linkMembership( user, group, mt, true);
      Object parent = uiForm.getParent() ;
      if(parent instanceof UIUserInfo) {
        UIUserInfo uiParent = (UIUserInfo) parent ;
        uiParent.update();
      } else if(parent instanceof UIViewUserInGroup) {
        UIViewUserInGroup uiParent = (UIViewUserInGroup) parent ;
        uiParent.update() ;
      }
    }
  }
  static public class SelectUserListener extends ExoActionListener  {
    public SelectUserListener( ) {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIMembershipForm uiForm = (UIMembershipForm) event.getSource() ;
      UIViewUserInGroup uiViewUserInGroup = (UIViewUserInGroup) uiForm.getParent();
      uiViewUserInGroup.setShowMembership(false);
      uiForm.setRenderedSibling(UISelectUserManager.class) ;
    }
  }
}