/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.admin;

import condition.* ;
import suites.share.CommonUnit ;
import suites.share.UserLoginSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.share.UserLogoutSuite;

/**
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Author : Hoa Nguyen
 *          hoa2501@yahoo.com
 * Mar 08, 2005
 */
public class AdminViewSuite extends WebUnitSuite {
  static String DESC = "Go to Manager Admin page and test all basic actions" ;
  static String NAME = "AdminViewSuite" ;
  String COMMENT = null;   
  public AdminViewSuite() {    
    super(NAME, DESC) ;     
    goToPage("/Home/ecm") ;
    //Click Admin The ECM
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTheECM", "Select Admin The ECM tab").
      setURL("componentId=ECMPage&portal:action=changeTab&objectId=ecm").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=addEditView"))      
    ) ;      
    //Click on Manager View icon
    addWebUnit(
      new ClickURLLinkUnit("ClickManagerView", "Click on Manage View icon").
      setURL("&uicomponent=UIECMAdminSummary&op=addEditView").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewList&op=add"))      
    ) ;  
    //-----Add a new template
    addWebUnit(
        new ClickURLLinkUnit("ClickOnTemp", "Click on template icon").
        setURL("uicomponent=UINode&op=selectNode&objectId=ecmTemplatelist").   
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIECMTemplateList&op=add"))      
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("ClickAddNewTemp", "Click on add template icon").
        setURL("uicomponent=UIECMTemplateList&op=add").   
        addValidator(new ExpectFormIdValidator("UITemplateForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("CancelAddNewTemp", "cancel add a new template").
        setFormId("UITemplateForm").    
        setField("op","PhaseId[1].cancel").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIECMTemplateList&op=add"))      
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("ClickAddNewTemp", "Click on add template icon").
        setURL("uicomponent=UIECMTemplateList&op=add").   
        addValidator(new ExpectFormIdValidator("UITemplateForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("AddNewTempNameEmpty", "add new template with name is empty").
        setFormId("UITemplateForm"). 
        setField("name","").
        setField("op","save").
        addValidator(new ExpectTextValidator("The field \"name\" cannot be empty"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("AddNewTempNameInv", "add new template with name is character special").
        setFormId("UITemplateForm"). 
        setField("name","#").
        setField("op","save").
        addValidator(new ExpectTextValidator("Only alpha, digit and underscore characters allowed for the field name"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("AddNewTempValid", "add new template is valid").
        setFormId("UITemplateForm"). 
        setField("name","Template-test").
        setField("op","save").
        addValidator(new ExpectTextValidator("Template-test"))      
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("ClickEditNewTemp", "Click on edit template icon").
        setURL("uicomponent=UIECMTemplateList&op=edit&objectId=/jcr:system/exo:ecm/views/templates/ecm-explorer/Template-test").                
        addValidator(new ExpectTextValidator("Template-test"))      
    ) ;
    addWebUnit(
        new SubmitFormUnit("EditNewTemp", "edit new template").
        setFormId("UITemplateForm"). 
        setField("content","Hello").
        setField("op","save").
        addValidator(new ExpectTextValidator("Template-test"))      
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("DeleteNewTemplate", "Click on delete template icon of Template-test").
        setURL("uicomponent=UIECMTemplateList&op=delete&objectId=/jcr:system/exo:ecm/views/templates/ecm-explorer/Template-test").        
        addValidator(new ExpectNoTextValidator("Template-test"))      
    ) ;
    //-------------------------------
    addWebUnit(
        new ClickURLLinkUnit("ClickCBTemplates", "Click on CBTemplates tab").
        setURL("uicomponent=UINode&op=selectNode&objectId=cbTemplatelist").   
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICBTemplateList&op=add"))      
    ) ;
    //-----Add a new content browse
    addWebUnit(
        new ClickURLLinkUnit("DeleteCBtest-path", "Click on delete CBTemplate icon of test-path").
        setURL("uicomponent=UICBTemplateList&op=delete&objectId=/jcr:system/exo:ecm/views/templates/content-browser/path/test-path").    
        addCondition(new HaveLinkWithURLCondition("uicomponent=UICBTemplateList&op=delete&objectId=/jcr:system/exo:ecm/views/templates/content-browser/path/test-path")).
        addValidator(new ExpectNoTextValidator("test-path"))      
    ) ;   
    addWebUnit(
        new ClickURLLinkUnit("ClickAddCBTemplates", "Click on add icon to add a new CBTemplates").
        setURL("uicomponent=UICBTemplateList&op=add").   
        addValidator(new ExpectFormIdValidator("UITemplateForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("CreateCBtest-path", "Click on save icon to add new test-path CBTemplate").
        setFormId("UITemplateForm"). 
        setField("content","test-path").
        setField("name","test-path").
        setField("homeTemplate","/jcr:system/exo:ecm/views/templates/content-browser/path").
        setField("op","save").
        addValidator(new ExpectTextValidator("test-path"))      
    ) ; 
    //-----Add a new CBTemplate test-query
    addWebUnit(
        new ClickURLLinkUnit("DeleteCBtest-query", "Click on delete CBTemplate icon of test-query").
        setURL("uicomponent=UICBTemplateList&op=delete&objectId=/jcr:system/exo:ecm/views/templates/content-browser/query/test-query").    
        addCondition(new HaveLinkWithURLCondition("uicomponent=UICBTemplateList&op=delete&objectId=/jcr:system/exo:ecm/views/templates/content-browser/query/test-query")).
        addValidator(new ExpectNoTextValidator("test-query"))      
    ) ;   
    addWebUnit(
        new ClickURLLinkUnit("ClickAddCBTemplates", "Click on add icon to add a new CBTemplates").
        setURL("uicomponent=UICBTemplateList&op=add").   
        addValidator(new ExpectFormIdValidator("UITemplateForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("CreateCBtest-query", "Click on save icon to add new test-query CBTemplate").
        setFormId("UITemplateForm"). 
        setField("content","test-query").
        setField("name","test-query").
        setField("homeTemplate","/jcr:system/exo:ecm/views/templates/content-browser/query").
        setField("op","save").
        addValidator(new ExpectTextValidator("test-query"))      
    ) ; 
    //-----Add a new CBTemplate test-script
    addWebUnit(
        new ClickURLLinkUnit("DeleteCBtest-script", "Click on delete CBTemplate icon of test-script").
        setURL("uicomponent=UICBTemplateList&op=delete&objectId=/jcr:system/exo:ecm/views/templates/content-browser/script/test-script").    
        addCondition(new HaveLinkWithURLCondition("uicomponent=UICBTemplateList&op=delete&objectId=/jcr:system/exo:ecm/views/templates/content-browser/script/test-script")).
        addValidator(new ExpectNoTextValidator("test-script"))      
    ) ;   
    addWebUnit(
        new ClickURLLinkUnit("ClickAddCBTemplates", "Click on add icon to add a new CBTemplates").
        setURL("uicomponent=UICBTemplateList&op=add").   
        addValidator(new ExpectFormIdValidator("UITemplateForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("CreateCBtest-script", "Click on save icon to add new test-script CBTemplate").
        setFormId("UITemplateForm"). 
        setField("content","test-script").
        setField("name","test-script").
        setField("homeTemplate","/jcr:system/exo:ecm/views/templates/content-browser/script").
        setField("op","save").
        addValidator(new ExpectTextValidator("test-script"))      
    ) ; 
    //-----Add a new CBTemplate test-detailView
    addWebUnit(
        new ClickURLLinkUnit("DeleteCBtest-detailView", "Click on delete CBTemplate icon of test-detailView").
        setURL("uicomponent=UICBTemplateList&op=delete&objectId=/jcr:system/exo:ecm/views/templates/content-browser/detail-document/test-detailView").    
        addCondition(new HaveLinkWithURLCondition("uicomponent=UICBTemplateList&op=delete&objectId=/jcr:system/exo:ecm/views/templates/content-browser/detail-document/test-detailView")).
        addValidator(new ExpectNoTextValidator("test-detailView"))      
    ) ;   
    addWebUnit(
        new ClickURLLinkUnit("ClickAddCBTemplates", "Click on add icon to add a new CBTemplates").
        setURL("uicomponent=UICBTemplateList&op=add").   
        addValidator(new ExpectFormIdValidator("UITemplateForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("CreateCBtest-detailView", "Click on save icon to add new test-detailView CBTemplate").
        setFormId("UITemplateForm"). 
        setField("content","test-detailView").
        setField("name","test-detailView").
        setField("homeTemplate","/jcr:system/exo:ecm/views/templates/content-browser/detail-document").
        setField("op","save").
        addValidator(new ExpectTextValidator("test-detailView"))      
    ) ; 
    //Thang :0955666158
//    addWebUnit(
//        new ClickURLLinkUnit("ClickEditCBtest-path", "Click on edit icon to edit a new test-path CBTemplates").
//        setURL("uicomponent=UICBTemplateList&op=edit&objectId=/jcr:system/exo:ecm/views/templates/content-browser/path/test-path").       
//        addValidator(new ExpectFormIdValidator("UITemplateForm"))      
//    ) ; 
//    addWebUnit(
//        new SubmitFormUnit("EditCBTemplate", "Click on save icon to add new CBTemplate").
//        setFormId("UITemplateForm"). 
//        setField("content","test-path").
//        setField("name","CBTemplate-test").
//        setField("type","cbQueryTemplates").
//        setField("op","save").
//        addValidator(new ExpectTextValidator("CBTemplate-test"))      
//    ) ; 
    //---------------------------------
    addWebUnit(
        new ClickURLLinkUnit("ClickOnView", "Click on view icon").
        setURL("&uicomponent=UINode&op=selectNode&objectId=viewlist").   
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewList&op=add"))      
    ) ;   
    //-----Add a new view
    addWebUnit(
      new ClickURLLinkUnit("AddView", "Click on add view icon").
      setURL("&uicomponent=UIViewList&op=add").                  
      addValidator(new ExpectFormIdValidator("uiViewForm"))  
    ) ;
    addWebUnit(
        new SubmitFormUnit("SaveViewNameEmpty", "Click on save button").
        setFormId("uiViewForm").    
        setField("name","").
        setField("userOrGroup","*:/user").
        setField("op","save").
        addValidator(new ExpectTextValidator("View name not be empty!"))  
    ) ; 
    addWebUnit(
        new SubmitFormUnit("SaveViewPermissionsInv", "Click on save button with permissions field invalid").
        setFormId("uiViewForm").    
        setField("name","view-test").
        setField("userOrGroup","#").
        setField("op","save").
        addValidator(new ExpectTextValidator("Invalid permission, permission style should be membership:/groupId."))  
    ) ;  
    addWebUnit(
        new SubmitFormUnit("SaveViewPermissionsInv", "Click on save button with Membership type or Group name not found").
        setFormId("uiViewForm").    
        setField("name","view-test").
        setField("userOrGroup","*:/userfault").
        setField("op","save").
        addValidator(new ExpectTextValidator("Membership type or Group name not found."))  
    ) ;  
    addWebUnit(
        new SubmitFormUnit("SearchPermissions", "Click on Search icon").
        setFormId("uiViewForm").    
        setField("op","searchGroup").
        addValidator(new ExpectTextValidator("Group Name"))  
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("CancelSelectPer", "Click on back icon").
        setURL("uicomponent=UIExoPermissionBrowser&op=PhaseId[1].cancel").   
        addValidator(new ExpectFormIdValidator("uiViewForm"))   
    ) ; 
    addWebUnit(
        new SubmitFormUnit("SearchPermissions", "Click on Search icon").
        setFormId("uiViewForm").    
        setField("op","searchGroup").
        addValidator(new ExpectTextValidator("Group Name"))  
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("SelectAnyUser", "Select any user").
        setURL("uicomponent=UIExoPermissionBrowser&op=selectPermission&objectId=*:/user").   
        addValidator(new ExpectFormIdValidator("uiViewForm"))   
    ) ; 
     addWebUnit(
        new SubmitFormUnit("SaveNewViewNotTab", "Click on save button").
        setFormId("uiViewForm").    
        setField("name","view-test").
        setField("userOrGroup","*:/user").
        setField("op","save").
        addValidator(new ExpectTextValidator("View view-test must have one or any tabs"))  
    ) ;  
    //----------Add Admin tab
    addWebUnit(
      new SubmitFormUnit("AddAdminTab", "Click on add tab icon").
      setFormId("uiViewForm").    
      setField("op","addTab").
      addValidator(new ExpectFormIdValidator("uiTabForm"))  
    ) ;        
    addWebUnit(
        new SubmitFormUnit("SubmitNotAnyButton", "not select any button and click save").
        setFormId("uiTabForm").    
        setField("name","Admin").
        setField("op","save").
        addValidator(new ExpectTextValidator("Can not save tab without select button!"))  
    ) ;  
    addWebUnit(
      new SubmitFormUnit("SubmitTabNameEmpty", "Select any button and click save").
      setFormId("uiTabForm").    
      setField("name","").
      setCheckBox("manageVersions",true).
      setCheckBox("manageCategories",true).
      setCheckBox("manageActions",true).
      setCheckBox("manageRelations",true).
      setCheckBox("exportNode",true).
      setCheckBox("importNode",true).
      setField("op","save").
      addValidator(new ExpectTextValidator("Tab name not be empty!"))  
    ) ;  
    addWebUnit(
        new SubmitFormUnit("SubmitAdminTabValid", "Add a tab has name is Admin with some button").
        setFormId("uiTabForm").    
        setField("name","Admin").
        setField("op","save").
        addValidator(new ExpectFormIdValidator("uiViewForm"))  
    ) ;  
    //----------Add Info tab
    addWebUnit(
      new SubmitFormUnit("AddInfoTab", "Click on add tab icon").
      setFormId("uiViewForm").    
      setField("op","addTab").
      addValidator(new ExpectFormIdValidator("uiTabForm"))  
    ) ;    
    addWebUnit(
      new SubmitFormUnit("SubmitInfoTab", "Add a tab has name is Info with some button").
      setFormId("uiTabForm").  
      setField("name","Info").
      setCheckBox("viewReferences",true).
      setCheckBox("viewNodeType",true).
      setCheckBox("viewPermissions",true).
      setCheckBox("viewProperties",true).  
      setCheckBox("viewRelations",true).  
      setField("op","save").
      addValidator(new ExpectFormIdValidator("uiViewForm"))  
    ) ;
    //----------Add Action tab
    addWebUnit(
      new SubmitFormUnit("AddActionTab", "Click on add tab icon").
      setFormId("uiViewForm").    
      setField("op","addTab").
      addValidator(new ExpectFormIdValidator("uiTabForm"))  
    ) ;    
    addWebUnit(
      new SubmitFormUnit("SubmitActionsTab", "Add a tab has name is Actions with some button").
      setFormId("uiTabForm").    
      setField("name","Actions").
      setCheckBox("addFolder",true).
      setCheckBox("addDocument",true).
      setCheckBox("upload",true).
      setCheckBox("createTicket",true).
      setCheckBox("paste",true).
      setCheckBox("search",true).
      setField("op","save").
      addValidator(new ExpectFormIdValidator("uiViewForm"))  
    ) ;   
    addWebUnit(
      new SubmitFormUnit("SaveNewView", "Click on save button").
      setFormId("uiViewForm").    
      setField("name","view-test").
      setField("userOrGroup","*:/user").
      setField("template","/jcr:system/exo:ecm/views/templates/ecm-explorer/ThumbnailsView").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIViewList&op=edit&objectId=view-test"))  
    ) ;  
    //-----Edit view-test
    addWebUnit(
        new ClickURLLinkUnit("EditView", "Click on edit icon").
        setURL("uicomponent=UIViewList&op=edit&objectId=view-test").
        addValidator(new ExpectFormIdValidator("uiViewForm"))   
    ) ; 
    addWebUnit(
        new SubmitFormUnit("SaveEditView", "Click on save button").
        setFormId("uiViewForm").    
        setField("name","view-test").
        setField("userOrGroup","*:/user").
        setField("template","/jcr:system/exo:ecm/views/templates/ecm-explorer/ListView").
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewList&op=add"))  
    ) ;  
    //-----Click on back
    addWebUnit(
        new ClickURLLinkUnit("ClickOnBack", "Click on Back icon").
        setURL("uicomponent=UIViewList&op=back").                  
        addValidator(new ExpectTextValidator("Manage View"))      
    ) ; 
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }  
}