/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;


/**
 * Test copy, cut and paste actions on newly created documents. This Suite
 * depends on the AddDocumentSuite one.      
 *     1) Go to production workspace
 *     2) Go to /test-folder
 *     3) Create a /test-folder2 directory
 *     4) Copy test-doc
 *     5) Go to /test-folder2
 *     6) Paste test-doc and check it is added
 *     7) Go back to /test-folder
 *     8) Create /test-folder3
 *     9) Cut test-doc
 *    10) Go to test-folder3
 *    11) Paste test-doc and check it is added
 *    12) Go back to test-folder
 *    13) Check test-doc was deleted
 *    14) Save Session
 *    15) Go back to workspaces selection page
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class UserLockCopyPasteCutSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page, click on test-folder and to do somethings" ;
  static String NAME = "UserLockCopyPasteCutSuite" ;
    
  public UserLockCopyPasteCutSuite() {    
    super(NAME, DESC) ;   
    addWebSuite(new UserAddDocumentSuite());      
    //Go to production 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnProduction", "click on Production button").
      setURL("&uicomponent=UIWorkspaceBrowser&op=selectWorkspace&objectId=production").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectWorkspace"))      
    ) ;
    //-----Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").       
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ; 
    //-----Goto /home/users/#{client.name}
   addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "click on home link").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users"))     
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUsers", "click on users link").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnCurrentUser", "click on current user link").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}")
    ) ;
//    addWebUnit(
//        new ClickURLLinkUnit("LockTest-folder", "click on lock icon of row test-folder").
//        setURL("uicomponent=UIDocumentInfo&op=lock&objectId=test-folder").
//        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=unlock&objectId=test-folder"))
//    ) ;
//    addWebUnit(
//        new ClickURLLinkUnit("CopyLockedTest-folder", "Click Copy icon of locked test-folder").
//        setURL("&uicomponent=UIDocumentInfo&op=copy&objectId=test-folder").
//        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=unlock&objectId=test-folder"))
//    ) ;
    //Select the test-folder
    addWebUnit(
      new ClickURLLinkUnit("GotoTestfolder", "click on test-folder").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder").         
      addValidator(new ExpectTextValidator(">")).
      addValidator(new ExpectTextValidator("test-folder"))
    ) ; 
    //Create a /test-folder2 directory    
    addWebUnit(
      new ClickURLLinkUnit("DeleteIfExistedTestfolder2", "click delete button if test-folder2 existed").
      setURL("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder/test-folder2").      
      addValidator(new ExpectTextValidator("The node /test-folder2 has been removed.")).
      addCondition(new HaveLinkWithURLCondition("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder/test-folder2"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("CreateTestfolder2", "click on add folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").         
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;    
    addWebUnit(
      new SubmitFormUnit("SaveNewFolder", "click on Save button").
      setFormId("folderForm").   
      setField("name","test-folder2").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder/test-folder2"))                             
    ) ;
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;    
    //Copy test-doc
    addWebUnit(
      new ClickURLLinkUnit("CopyDocument", "click on copy button").
      setURL("uicomponent=UIDocumentInfo&op=copy&objectId=/home/users/#{client.name}/test-folder/test-doc").     
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder/test-doc has been copied."))     
    ) ;   
    //Go to /test-folder2
    addWebUnit(
      new ClickURLLinkUnit("GotoTest-folder2", "click on test-folder2").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder/test-folder2").        
      addValidator(new ExpectTextValidator(">")).
      addValidator(new ExpectTextValidator("test-folder")).      
      addValidator(new ExpectTextValidator("test-folder2"))
    ) ;       
    //Paste test-doc and check it is added
    addWebUnit(
      new ClickURLLinkUnit("PasteDocument", "click on paste button").
      setURL("&uicomponent=UIToolbar&op=paste").        
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder/test-doc has been pasted at the location /home/users/#{client.name}/test-folder/test-folder2/test-doc. This operation has already been save and you do not need to press the floppy icon."))      
    ) ;        
    //Go back to /test-folder
    addWebUnit(
      new ClickURLLinkUnit("BackToTest-folder", "click on test-folder link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/home/users/#{client.name}/test-folder").        
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/home"))                            
    ) ; 
    //Create /test-folder3
    addWebUnit(
      new ClickURLLinkUnit("DeleteIfExistedTest-folder3", "click delete button if test-folder3 existed").
      setURL("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder/test-folder3").      
      addValidator(new ExpectTextValidator("The node /test-folder3 has been removed.")).
      addCondition(new HaveLinkWithURLCondition("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder/test-folder3"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("CreateTest-folder3", "click on add folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").         
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewFolder", "click on Save button").
      setFormId("folderForm").   
      setField("name","test-folder3").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder/test-folder3"))      
    ) ;
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;
    //Cut test-doc
    addWebUnit(
      new ClickURLLinkUnit("CutDocument", "click on cut button").
      setURL("uicomponent=UIDocumentInfo&op=cut&objectId=/home/users/#{client.name}/test-folder/test-doc").         
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder/test-doc has been cut."))         
    ) ;
    //Go to test-folder3
    addWebUnit(
      new ClickURLLinkUnit("GotoTest-folder3", "click on test-folder3").
      setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder/test-folder3").      
      addValidator(new ExpectTextValidator(">")).
      addValidator(new ExpectTextValidator("test-folder")).
      addValidator(new ExpectTextValidator("test-folder3"))
    ) ;    
    //Paste test-doc and check it is added    
    addWebUnit(
      new ClickURLLinkUnit("PasteDocument", "click on paste button").
      setURL("&uicomponent=UIToolbar&op=paste").
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder/test-doc has been pasted at the location /home/users/#{client.name}/test-folder/test-folder3/test-doc. This operation has already been save and you do not need to press the floppy icon."))
    ) ;
    //Go back to test-folder and Check test-doc was deleted              
    addWebUnit(
      new ClickURLLinkUnit("BackToTest-folder", "click on test-folder link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/home/users/#{client.name}/test-folder").     
      addValidator(new ExpectNoTextValidator("test-doc"))      
    ) ;    
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;      
    //Return to workspace selection
    addWebUnit(
      new ClickURLLinkUnit("SelectWorkspace", "select Workspace button").
      setURL("&uicomponent=MainToolbar&op=selectWorkspace").                  
      addValidator(new ExpectTextValidator("Workspaces "))      
    ) ;    
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}