/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;


/**
 * This suite does :
 *    1)Go to current user
 *    2)Add new folder has name is test-folder
 *    3)Add new document with type is exo:article which has name is test-doc
 *    4)Add test-doc to News category on cms
 *    5)Login with account exoadmin and add relationDoc document with type is exo:article  to cms/publication
 *    6)Check some status view relation
 *    7)Go back current user and add relation is relationDoc for test-doc
 *    8)Login with account exoadmin and go to browse category to check exist status of test-doc and relationDoc on News category
 *    9)Go back current user to delete relation,test-doc,test-folder
 *   10)Login with account exoadmin and delete relationDoc
 *   11)Go back current user
 *   
 * Created by The eXo Platform SARL
 * Author :Hoa Nguyen Thi 
 *         hoa2501@yahoo.com
 * Apr 12, 2006
 */
public class UserRelationSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page, click on test-folder and to do somethings" ;
  static String NAME = "UserRelationSuite" ;
  public UserRelationSuite() {    
    super(NAME, DESC) ; 
    //*Add new folder
    addWebSuite(new UserAddFolderSuite());
    //*Add new Doc has type is nt:file
    addWebUnit(
        new ClickURLLinkUnit("AddNewDocument", "Click on add document icon").
        setURL("uicomponent=UIToolbar&op=addDocument").               
        addValidator(new ExpectFormIdValidator("documentForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("SelectDocumentType", "select document type is exo:article").
        setFormId("selectDocumentForm").   
        setField("documentType","exo:article").
        setField("op","changeDocument").
        addValidator(new ExpectTextValidator("Image to upload"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SaveNewDocument", "click on save button").
        setFormId("documentForm").   
        setField("docName","test-doc").
        setField("title","test-doc").
        setField("op","save").
        addValidator(new ExpectTextValidator("test-doc"))
    ) ;
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;
    //*Add test-doc to News category on cms
    addWebUnit(
        new ClickURLLinkUnit("ClickOntest-doc", "Click on test-doc").
        setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder/test-doc").        
        addValidator(new ExpectTextValidator("test-doc"))      
    ) ;
    //-----Select Admin tab
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTab", "Select Admin tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Admin").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageVersions")).      
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageCategories")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageActions")).      
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=exportNode")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=importNode"))
    ) ;   
    //-----Manage Category
    addWebUnit(
        new ClickURLLinkUnit("ClickOnManageCategory", "Click on Manage Category icon").
        setURL("uicomponent=UIToolbar&op=manageCategories").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoriesList&op=add"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickAddCategory", "Click on Add Category icon").
        setURL("&uicomponent=UICategoriesList&op=add").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrCategoryBrowser&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/cms"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOncms", "Click on cms link").
        setURL("uicomponent=JcrCategoryBrowser&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/cms").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrCategoryBrowser&op=selectNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/cms/news"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("SelectNews", "Select News category").
        setURL("uicomponent=JcrCategoryBrowser&op=selectNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/cms/news").                  
        addValidator(new ExpectTextValidator("/jcr:system/exo:ecm/exo:taxonomies/cms/news"))
    ) ;
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("GoBacktest-doc", "Click on back icon to go back test-doc").
        setURL("uicomponent=UICategoriesList&op=PhaseId[1].cancel").                  
        addValidator(new ExpectTextValidator("test-doc"))
    ) ; 
    //-----Login with admin account
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
    );     
    String[] roles = ["user", "admin"] ;
    addWebSuite(new UserLoginSuite("exoadmin", "exo", roles)) ;   
    goToPage("/Home/ecm") ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnProduction", "click on Production button").
        setURL("&uicomponent=UIWorkspaceBrowser&op=selectWorkspace&objectId=production").                  
        addValidator(new ExpectLinkWithURLValidator("portal:isSecure=false&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnCms", "click on home cms").
        setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms").             
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications"))           
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPublications", "click on publications link").
        setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Actions"))
    ) ;
    //----------Add new Doc has type is exo:article
    addWebUnit(
        new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
        setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("AddNewDocument", "Click on add document icon").
        setURL("uicomponent=UIToolbar&op=addDocument").               
        addValidator(new ExpectFormIdValidator("documentForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("SelectDocumentType", "select document type is exo:article").
        setFormId("selectDocumentForm").   
        setField("documentType","exo:article").
        setField("op","changeDocument").
        addValidator(new ExpectTextValidator("Image to upload"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SaveNewDocument", "click on save button").
        setFormId("documentForm").   
        setField("docName","relationDoc").
        setField("title","relationDoc").
        setField("op","save").
        addValidator(new ExpectTextValidator("relationDoc"))
    ) ;
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("SelectInfoTab ", "Select Info tab").
        setURL("uicomponent=UIViewController&op=selectNode&objectId=Info").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=viewRelations"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnViewRelation", "Click on view relation").
        setURL("uicomponent=UIToolbar&op=viewRelations").        
        addValidator(new ExpectTextValidator("This node does'nt support relationable"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnrelationDoc", "Click on relationDoc").
        setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications/relationDoc").                
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=viewRelations"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnViewRelation", "Click on view relation").
        setURL("uicomponent=UIToolbar&op=viewRelations").        
        addValidator(new ExpectTextValidator("This node hasn't been relationable yet"))      
    ) ;
    //-----Back to current user
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
    );    
    addWebUnit(
        new ClickURLLinkUnit("SelectAdminTab", "Select Admin tab").
        setURL("uicomponent=UIViewController&op=selectNode&objectId=Admin").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=manageRelations"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("SelectManageRelations", "Select Manage Relation").
        setURL("uicomponent=UIToolbar&op=manageRelations").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIRelationsList&op=add"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnAddRelation", "Click on add relation icon").
        setURL("uicomponent=UIRelationsList&op=add").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrRelationBrowser&op=changeNode&objectId=/cms/publications"))      
    ) ;   
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPublications", "Click on add relation icon").
        setURL("uicomponent=JcrRelationBrowser&op=changeNode&objectId=/cms/publications").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrRelationBrowser&op=selectNode&objectId=/cms/publications/relationDoc"))      
    ) ;  
    addWebUnit(
        new ClickURLLinkUnit("SelectRelationsDoc", "Select relationDoc").
        setURL("uicomponent=JcrRelationBrowser&op=selectNode&objectId=/cms/publications/relationDoc").        
        addValidator(new ExpectTextValidator("/cms/publications/relationDoc"))      
    ) ;
    //Save session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnBack", "Click on back").
        setURL("uicomponent=UIRelationsList&op=PhaseId[1].cancel").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications/relationDoc"))      
    ) ; 
    addWebUnit(
        new ClickURLLinkUnit("ClickRelationDoc", "Click on relationDoc").
        setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications/relationDoc").        
        addValidator(new ExpectTextValidator("relationDoc"))      
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("GoBackHome", "Click on Home icon").
        setURL("uicomponent=UIJCRExplorer&op=changeNode&objectId=/").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=changeNode&objectId=/home"))      
    ) ;   
    //-----Login with admin account
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
    );     
    addWebSuite(new UserLoginSuite("exoadmin", "exo", roles)) ;   
    addWebUnit(
        new ClickLinkWithText("ClickBrowseContent", "Click on browse content").
        setTextLink("Browse Content").
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentList&op=viewCategory&objectId=/jcr:system/exo:ecm/exo:taxonomies/cms"))
    ) ;  
    addWebUnit(
        new ClickURLLinkUnit("Clickcms", "Click on cms  link").
        setURL("uicomponent=UIContentList&op=viewCategory&objectId=/jcr:system/exo:ecm/exo:taxonomies/cms").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentList&op=viewCategory&objectId=/jcr:system/exo:ecm/exo:taxonomies/cms/news"))     
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickNews", "Click on News  link").
        setURL("uicomponent=UIContentList&op=viewCategory&objectId=/jcr:system/exo:ecm/exo:taxonomies/cms/news").                  
        addValidator(new ExpectLinkWithURLValidator("test-folder/test-doc"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOntest-doc", "Click on test-doc link").
        setURL("test-folder/test-doc").                
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDetailDocument&op=viewRelated&objectId=production:/cms/publications/relationDoc"))            
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnrelationDoc", "Click on test-doc link").
        setURL("uicomponent=UIDetailDocument&op=viewRelated&objectId=production:/cms/publications/relationDoc").
        addValidator(new ExpectTextValidator("relationDoc"))            
    ) ;      
    //-----Back to current user
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
    );    
    //-----Goto /home/users/#{client.name}
    addWebUnit(
       new ClickURLLinkUnit("ClickOnHome", "click on home link").
       setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home").                  
       addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users"))     
     ) ; 
     addWebUnit(
       new ClickURLLinkUnit("ClickOnUsers", "click on users link").
       setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users").                  
       addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}"))      
     ) ;
     addWebUnit(
       new ClickURLLinkUnit("ClickOnCurrentUser", "click on current user link").
       setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}")
     ) ;
     //-----Go to test-folder
     addWebUnit(
       new ClickURLLinkUnit("GotoTest-folder", "click on test-folder").
       setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder").         
       addValidator(new ExpectTextValidator(">")).
       addValidator(new ExpectTextValidator("test-folder"))
     ) ;     
     addWebUnit(
         new ClickURLLinkUnit("SelectTest-doc ", "Click on test-doc link").
         setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder/test-doc").        
         addValidator(new ExpectLinkWithURLValidator("uicomponent=UIViewController&op=selectNode&objectId=Admin"))      
     ) ;
     addWebUnit(
         new ClickURLLinkUnit("SelectInfoTab ", "Select Info tab").
         setURL("uicomponent=UIViewController&op=selectNode&objectId=Info").        
         addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=viewRelations"))      
     ) ;
     addWebUnit(
         new ClickURLLinkUnit("ClickOnViewRelation ", "Click on view relation").
         setURL("uicomponent=UIToolbar&op=viewRelations").        
         addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications/relationDoc"))      
     ) ;
     addWebUnit(
         new ClickURLLinkUnit("ClickOnrelationDoc ", "Click on relationDoc").
         setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications/relationDoc").                                  
         addValidator(new ExpectTextValidator("relationDoc"))      
     ) ;
     addWebUnit(
         new ClickURLLinkUnit("ClickOnBackToPrevious ", "Click on back to previous").
         setURL("&uicomponent=MainToolbar&op=back").        
         addValidator(new ExpectTextValidator("test-doc"))      
     ) ;     
     addWebUnit(
         new ClickURLLinkUnit("SelectAdminTab", "Select Admin tab").
         setURL("uicomponent=UIViewController&op=selectNode&objectId=Admin").        
         addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=manageRelations"))      
     ) ;
     addWebUnit(
         new ClickURLLinkUnit("SelectManageRelations", "Select Manage Relation").
         setURL("uicomponent=UIToolbar&op=manageRelations").        
         addValidator(new ExpectLinkWithURLValidator("uicomponent=UIRelationsList&op=add"))      
     ) ;
     //----------Delete relation
     addWebUnit(
         new ClickURLLinkUnit("DeleteRelation", "Click on delete relation").
         setURL("uicomponent=UIRelationsList&op=delete&objectId=/cms/publications/relationDoc").        
         addValidator(new ExpectNoTextValidator("/cms/publications/relationDoc"))      
     ) ;
     //Save session
     addWebUnit(
       new ClickURLLinkUnit("SavedSession", "click on save Session button").
       setURL("&uicomponent=MainToolbar&op=saveSession").        
       addValidator(new ExpectTextValidator("All the changes have been saved"))      
     ) ;
     addWebUnit(
         new ClickURLLinkUnit("SelectInfoTab ", "Select Info tab").
         setURL("uicomponent=UIViewController&op=selectNode&objectId=Info").        
         addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=viewRelations"))      
     ) ;
     addWebUnit(
         new ClickURLLinkUnit("ClickOnViewRelation ", "Click on view relation").
         setURL("uicomponent=UIToolbar&op=viewRelations").        
         addValidator(new ExpectTextValidator("No related document"))      
     ) ;     
     addWebUnit(
         new ClickURLLinkUnit("GoToCurrentUser", "Go to current user").
         setURL("uicomponent=UIJCRExplorer&op=changeNode&objectId=/home/users/#{client.name}").        
         addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder"))      
     ) ;     
     //----------Delete test-folder
     addWebUnit(
         new ClickURLLinkUnit("Deletetest-folder", "Delete test-folder").
         setURL("uicomponent=UIDocumentInfo&op=removeNode&objectId=/home/users/#{client.name}/test-folder").                 
         addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder has been removed."))      
     ) ;
     //-----Login with admin account
     addWebUnit(
       new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
     );     
     addWebSuite(new UserLoginSuite("exoadmin", "exo", roles)) ;   
     //----------Goto Workflow and to do something
     goToPage("/Home/ecm") ;
     addWebUnit(
         new ClickURLLinkUnit("ClickOnProduction", "click on Production button").
         setURL("&uicomponent=UIWorkspaceBrowser&op=selectWorkspace&objectId=production").                  
         addValidator(new ExpectLinkWithURLValidator("portal:isSecure=false&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home"))      
     ) ;
     addWebUnit(
         new ClickURLLinkUnit("ClickOnCms", "click on home cms").
         setURL("uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms").             
         addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications"))           
     ) ;
     addWebUnit(
         new ClickURLLinkUnit("ClickOnPublications", "click on publications link").
         setURL("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/cms/publications").
         addValidator(new ExpectLinkWithURLValidator("uicomponent=UIDocumentInfo&op=removeNode&objectId=/cms/publications/relationDoc"))
     ) ;
     //----------Delete relationDoc
     addWebUnit(
         new ClickURLLinkUnit("DeleterelationDoc", "Click on delete icon on row of relationDoc").
         setURL("uicomponent=UIDocumentInfo&op=removeNode&objectId=/cms/publications/relationDoc"). 
                 
         addValidator(new ExpectTextValidator("The node /cms/publications/relationDoc has been removed."))
     ) ;
     //Save session
     addWebUnit(
       new ClickURLLinkUnit("SavedSession", "click on save Session button").
       setURL("&uicomponent=MainToolbar&op=saveSession").        
       addValidator(new ExpectTextValidator("All the changes have been saved"))      
     ) ;
     //-----Back to current user
     addWebUnit(
       new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
     );    
     //Return to workspace selection
     addWebUnit(
       new ClickURLLinkUnit("SelectWorkspace", "select Workspace button").
       setURL("&uicomponent=MainToolbar&op=selectWorkspace").                  
       addValidator(new ExpectTextValidator("Workspaces "))      
     ) ;  
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}