/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.share ;

import condition.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import org.exoplatform.test.portlet.Login.*;

/**
 * Created by The eXo Platform SARL
 * Author : Hoa  Pham
 *          hoapham@exoplatform.com,phamvuxuanhoa@yahoo.com
 * Nov 8, 2005
 */
public class PageBrowseSuite extends WebUnitSuite {
  String COMMENT="You must run PortletRegistrySuite suite before you run this suite";
  public PageBrowseSuite() {
    super("PageBrowseSuite", "Click browse page and do some action") ;        
    //------------------Create New Page with name is 'testPage'-------------------------
    addWebUnit(
      new ClickURLLinkUnit("ClickNewPage", "Tell web client click 'NewPage' on a page").
      setURL("uicomponent=UIToolbar&op=newPage").
      addValidator(new ExpectFormIdValidator("pageModelForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("Create testPage", "Fill page's information and create this page").
      setFormId("pageModelForm").
      setField("name","testPage").
      setField("template", "blank-page").
      setField("op", "save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=preview&objectId=#{client.name}:testPage")) 
    );            
    //--------------Click Preview on page---------------
    addWebUnit(
      new ClickURLLinkUnit("ClickPreview", "Tell web client click 'Preview' on a page").
      setURL("uicomponent=UIPageList&op=preview&objectId=#{client.name}:testPage").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPagePreviewToolbar&op=view"))
    ) ;
    addWebSuite(new PreviewPageSuite("#{client.name}","testPage")) ;
    //--------------Search Permission--------------
    //-------------------Have no results
    addWebUnit(
      new SubmitFormUnit("SearchWithInvalidOwner", "Search page with an invalid owner name").
      setFormId("searchForm").
      setField("owner","InvalidOwner").
      setField("viewPermission","").
      setField("editPermission","").
      setField("op","searchPages").      
      addValidator(new ExpectNoTextValidator("testPage"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchWithInvalidViewPer", "Search page with an invalid View Permission").
      setFormId("searchForm").
      setField("owner","#{client.name}").
      setField("viewPermission","InvalidViewPer:/user").
      setField("editPermission","").
      setField("op","searchPages").      
      addValidator(new ExpectNoTextValidator("testPage"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchWithInvalidEditPer", "Search page with a invalid Edit Permission").
      setFormId("searchForm").
      setField("owner","#{client.name}").
      setField("viewPermission","").
      setField("editPermission","InvalidEditPer:/user").
      setField("op","searchPages").      
      addValidator(new ExpectNoTextValidator("testPage"))
    ) ;
    //-------------------Have some results
    addWebUnit(
      new SubmitFormUnit("SearchWithOwner", "Search page with an valid owner name, all fields are blank").
      setFormId("searchForm").
      setField("owner","#{client.name}").
      setField("viewPermission","").
      setField("editPermission","").
      setField("op","searchPages").      
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=delete&objectId=#{client.name}:testPage"))
    ) ;
    //
    addWebUnit(
      new SubmitFormUnit("SearchWithViewPer(*:/user)", "Search page with a valid Owner and View Permission, all fields are blank").
      setFormId("searchForm").
      setField("owner","#{client.name}").
      setField("viewPermission","*:/user").
      setField("editPermission","").
      setField("op","searchPages").      
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=delete&objectId=#{client.name}:testPage"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchWithViewPer(:/user)", "Search page with a valid Owner and View Permission, all fields are blank").
      setFormId("searchForm").
      setField("owner","#{client.name}").
      setField("viewPermission",":/user").
      setField("editPermission","").
      setField("op","searchPages").      
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=delete&objectId=#{client.name}:testPage"))
    ) ;
    //
    addWebUnit(
      new SubmitFormUnit("SearchWithEditPer(*:/user)", "Search page with a valid Owner and Edit Permission, all fields are blank").
      setFormId("searchForm").
      setField("owner","#{client.name}").
      setField("viewPermission","").
      setField("editPermission","*:/user").
      setField("op","searchPages").      
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=delete&objectId=#{client.name}:testPage"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchWithEditPer(:/user)", "Search page with a valid Owner and Edit Permission, all fields are blank").
      setFormId("searchForm").
      setField("owner","#{client.name}").
      setField("viewPermission","").
      setField("editPermission",":/user").
      setField("op","searchPages").      
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=delete&objectId=#{client.name}:testPage"))
    ) ;
    //
    addWebUnit(
      new SubmitFormUnit("SearchWithAll(*:/user)", "Search page with a valid Owner, View Permission and Edit Permission").
      setFormId("searchForm").
      setField("owner","#{client.name}").
      setField("viewPermission","*:/user").
      setField("editPermission","*:/user").
      setField("op","searchPages").      
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=delete&objectId=#{client.name}:testPage"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchWithAll(:/user)", "Search page with a valid Owner, View Permission and Edit Permission").
      setFormId("searchForm").
      setField("owner","#{client.name}").
      setField("viewPermission",":/user").
      setField("editPermission",":/user").
      setField("op","searchPages").      
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=delete&objectId=#{client.name}:testPage"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("ReSearchWithOwner", "Re search page with an valid owner name, all fields are blank").
      setFormId("searchForm").
      setField("owner","#{client.name}").
      setField("viewPermission","").
      setField("editPermission","").
      setField("op","searchPages").      
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPageList&op=delete&objectId=#{client.name}:testPage"))
    ) ;
    //--------------Return Edit Mode---------------
    addWebUnit(
        new ClickURLLinkUnit("ReturnEditMode", "Tell web client click 'back' on a page").
        setURL("uicomponent=UIToolbar&op=return")        
    ) ;
  }
}